#!/usr/bin/env python

#program to play simulations from database
import MySQLdb, dgraph, cPickle
import visual as V
from math import *
from qt import *
from Numeric import *



class viewer:
    """
    """
    def __init__(self, host='localhost', port=3306, user='epigrass', pw='epigrass', db='epigrass'):
        self.host = host
        self.port = port
        self.user = user
        self.pw = pw
        self.db = db
        
        self.dmap = 0#int(input('Draw Map?(0,1) '))
        self.tables = self.getTables()

        #self.nodes = self.readNodes(self.tables[table])
        #self.numbnodes = len(self.nodes)
        #self.data = self.readData(self.tables[table])
        #self.numbsteps = len(self.data)/self.numbnodes
        #self.viewGraph()
        
    
    def getTables(self):
        """
        Returns a list of tables.
        """
        con=None
        try:
            con = MySQLdb.connect(host=self.host, port=self.port, user=self.user,passwd=self.pw, db=self.db)
            Cursor = con.cursor()
            Cursor.execute('SHOW TABLES')
            r=Cursor.fetchall()
        finally:
            if con:
                con.close()
                
        res = [i[0] for i in r]
        return res
    
    def readNodes(self,table):
        """
        Reads 1 row from data table and adjacency matrix.
        """
        con=None
        try:
            con = MySQLdb.connect(host=self.host, port=self.port, user=self.user,passwd=self.pw, db=self.db)
            Cursor = con.cursor()
            Cursor.execute('SELECT lat,longit FROM %s WHERE time = 0'%table)
            r=Cursor.fetchall()
            # get adjacency matrix
            file = open('adj_'+table)
            m = cPickle.load(file)
            file.close()
            
        finally:
            if con:
                con.close()
        return r,m
    
    
    def readData(self,table):
        """
        read time series data
        """
        con=None
        try:
            con = MySQLdb.connect(host=self.host, port=self.port, user=self.user,passwd=self.pw, db=self.db)
            Cursor = con.cursor()
            Cursor.execute('SELECT * FROM %s'%table)
            r=Cursor.fetchall()
        finally:
            if con:
                con.close()
        return r

    
    def viewGraph(self, nodes, am, mapa=''):
        """
        Starts the Vpython display of the graph.
        """
        self.gr = dgraph.Graph(0.04)
    
        Nlist = [dgraph.Node(3,(i[1],i[0],0)) for i in nodes]
        
        self.gr.insertNodeList(Nlist)
        el = self.gr.getEdgeFromMatrix(am)
        #print am,el
        El = [dgraph.RubberEdge(self.gr.nodes[c],self.gr.nodes[l],1,damping=0.8) for c,l in el]
        #print El
        self.gr.insertEdgeList(El)
        
        self.gr.centerView()
        if mapa != 'No map':
            if mapa!= 'Nenhum mapa':
                m = dgraph.Map(mapa)
                self.gr.insertMap(m)
            
    def anim(self,data, numbsteps, rate=20):
        """
        Starts the animation
        """
        for t in range(numbsteps):
            V.rate(rate)
            self.gr.timelabel.text=str(t) 
            for i,n in enumerate(self.gr.nodes):
                start = i*numbsteps+t
                n.box.width = n.box.height = n.box.length = (log10(data[start][6]+10))**1/3.
                if data[start][6] > 1:
                    self.paintNode(t,n,numbsteps)
    
    def paintNode(self,t,node,numbsteps,col='r'):
        """
        Paints red the sphere corresponding to the node in the visual display
        """
        if col =='r':
            if not node.painted: 
                red = (int(numbsteps)-t)/float(numbsteps)
                blue = 1-red**6
                node.box.color = (red,0.,blue)
                node.painted = 1
        elif col =='g':
            node.box.color=V.color.green


    def keyin(self,data,numbsteps,rate):
        while 1:

            if self.gr.display.kb.keys: # is there an event waiting to be processed?
        
                s = self.gr.display.kb.getkey() # obtain keyboard information
        
                if s == 'r':
                    for i in self.gr.nodes:
                        self.paintNode(0,i,numbsteps,col='g')
                        i.painted=0
                    self.anim(data,numbsteps,rate)
                else:
                    pass
        
                

if __name__ == "__main__":
    Display=viewer(user='root',pw='mysql')
    Display.anim()
    Display.keyin()
